import { PlottableData } from "./plottable-data";
import { PabResults } from "./pab-results";
import { GraphType } from "./graph-type";
import { ServiceFactory } from "../services/service-factory";

export class PlottablePabResults implements PlottableData {

    public graphType: GraphType = GraphType.Scatter;
    public chartX: string;
    public chartY: string;

    protected pabResults: PabResults;

    public constructor(pabResults?: PabResults) {
        if (pabResults) {
            this.setPabResults(pabResults);
        }
        // axes par défaut
        this.chartX = this.chartX || "CLOISON";
        this.chartY = this.chartY || "Z";
    }

    /** reaffect pabResults, for ex. when objet was contructed with empty pabResults */
    public setPabResults(pabResults: PabResults) {
        this.pabResults = pabResults;
    }

    /**
     * Returns the label to display, for an element of getAvailableChartAxis()
     * @param symbol parameter / result symbol (ex: "Q")
     */
    public getChartAxisLabel(symbol: string): string {
        return this.pabResults.headers[this.pabResults.columns.indexOf(symbol)];
    }

    public expandLabelFromSymbol(symbol: string): string {
        return symbol;
    }

    /**
     * Returns a list of plottable parameters / result elements, that can be defined
     * as X or Y chart axis
     */
    public getAvailableChartAxis(): string[] {
        return this.pabResults.columns;
    }

    // just to implement interface
    public getVariatingParametersSymbols(): string[] {
        return [];
    }

    /**
     * Returns the series of values for the required symbol
     * @param symbol parameter / result symbol (ex: "Q")
     */
    public getValuesSeries(symbol: string): any[] {
        const data: string[] = [];
        const pr = this.pabResults;
        const nDigits = ServiceFactory.instance.applicationSetupService.displayDigits;
        const l = this.pabResults.cloisonsResults.length;

        if (this.hasError()) {
            return [];
        }

        switch (symbol) {
            case "CLOISON":
                data.push("");
                for (let i = 0; i < l; i++) {
                    data.push("" + (i + 1));
                }
                data.push(ServiceFactory.instance.i18nService.localizeText("INFO_LIB_AVAL"));
                break;

            case "DH":
            case "ZRAM":
            case "Q":
                data.push("");
                for (let i = 0; i < l; i++) {
                    const er = pr.cloisonsResults[i].getExtraResult(symbol);
                    data.push((er !== undefined) ? er.toFixed(nDigits) : "");
                }
                const zrAval = pr.cloisonAvalResults.getExtraResult(symbol);
                data.push((zrAval !== undefined) ? zrAval.toFixed(nDigits) : "");
                break;

            case "Z":
                for (let i = 0; i < l - 1; i++) {
                    data.push(pr.cloisonsResults[i].vCalc.toFixed(nDigits));
                }
                data.push(pr.cloisonAvalResults.vCalc.toFixed(nDigits));
                data.push(pr.Z2.toFixed(nDigits));
                break;

            case "PV":
            case "YMOY":
            case "ZRMB":
            case "QA":
                data.push("");
                for (let i = 0; i < l; i++) {
                    const er = pr.cloisonsResults[i].getExtraResult(symbol);
                    data.push((er !== undefined) ? er.toFixed(nDigits) : "");
                }
                data.push("");
                break;
        }

        return data;
    }

    protected hasError(): boolean {
        return this.pabResults.hasError();
    }
}
